-- USDP Sample code
-- design to Test ADC/DAC Card
-- Feed through between ADC and DAC

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ADC_DAC_Feed is
    Port ( 	clk 		: in std_logic;		-- high speed clock
           	Rst 		: in std_logic;		-- system reset
           	CS 			: out std_logic;	-- ADC chip select
           	INT 		: in std_logic;		-- ADC Signal
           	Rd 			: out std_logic;	-- Sampling Clock for ADC   
		 	Test_clk	: out std_logic;	-- Test Clock
		 	Add			: out std_logic_vector(1 downto 0);		-- ADC  address lines
           	Data_ADC 	: in std_logic_vector(7 downto 0);		-- Data from ADC
           	DAC	 		: out std_logic_vector(11 downto 0);	-- DAC data bus         
           	En1,En2		: out std_logic);					    -- DAC latch enable
end ADC_DAC_Feed;

architecture Behavioral of ADC_DAC_Feed is
signal div				: std_logic_vector(10 downto 0);	-- clock divider
signal adc_latch		: std_logic_vector(7 downto 0);		-- ADC Latch
begin

-- DAC latch enable
En1			<= '0';
En2			<= '1';

-- ADC Chip select 
CS			<= '0';

-- Channel select of ADC
Add			<= "00";

-- Data to DAC; lower bits are padded with zeros to make the o/p of 12-bit
DAC	<= 	adc_latch & "0000";

------- clock divider  ---------------------------------
	process(clk,rst)
	begin
		if rst='0' then
			div	<= (others=>'0');
		elsif clk'event and clk='1' then
			div	<= div + 1;
		end if;
	end process;
--------------------------------------------------------
RD			<= div(6);
Test_clk	<= div(6);
------- DAC signals ----------------------------------

----- ADC latch ------------------
	process(rst,INT)
	begin
		if rst='0' then
			adc_latch <= (others=> '0');
		elsif INT'event and INT='0' then
		-- data coming from ADC is latched here
			adc_latch	<= Data_ADC;	
		end if;
	end process;

end Behavioral;
