-- Program to run led from one corner to another and count the number of rotation
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity running_LED is
    Port ( clk,rst 		: in std_logic;
           en 			: out std_logic_vector(3 downto 0);
           seg 			: out std_logic_vector(6 downto 0);
           led 			: out std_logic_vector(31 downto 0));
end running_LED;

architecture Behavioral of running_LED is

	COMPONENT seg_7
	PORT(
		bcd : IN std_logic_vector(3 downto 0);          
		seg : OUT std_logic_vector(6 downto 0)
		);
	END COMPONENT;
signal div 				: std_logic_vector(22 downto 0);
signal led_s			: std_logic_vector(31 downto 0);
signal bcd_s			: std_logic_vector(3 downto 0);
signal up,dwn			: std_logic;
signal en_s				: std_logic_vector( 3 downto 0);
signal cntr1,cntr2		: std_logic_vector(3 downto 0);
signal cntr3,cntr4		: std_logic_vector(3 downto 0);
signal en_clk			: std_logic;
signal dir1,dir2		: std_logic;
signal dir3,dir4		: std_logic;
signal led_clk			: std_logic;
begin

	u1: seg_7 PORT MAP(
		bcd => bcd_s,
		seg => seg
	);

bcd_s	<= cntr1 when en_s="0001" else
		   cntr2 when en_s="0010" else
		   cntr3 when en_s="0100" else
		   cntr4;

en		<= en_s;
en_clk  <= div(10);
led_clk	<= div(17);
led		<= led_s;
up		<= led_s(31);
dwn		<= led_s(0);


dir2	<= '1' when cntr1="0000" else
		   '0';
dir3	<= '1' when cntr2="0000" else
		   '0';
dir4	<= '1' when cntr3="0000" else
		   '0';

--- Clock divider
process(clk,rst)
begin
	if rst='0' then 
		div	<= (others=>'0');
	 elsif clk'event and clk='1' then
    	div <= div +1;
    end if ;
end process;


-- Running LED Code
process(led_clk,rst)
begin
	if rst='0' then
		led_s	<=(0=>'1', others=>'0');
	 elsif led_clk'event and led_clk='1' then
	 	if dir1='1' then
			led_s	<= led_s(30 downto 0) & led_s(31);	
		else
			led_s	<= led_s(0) & led_s(31 downto 1);
		end if;
	end if;
end process;

-- Changing the direction of RUN
	process(up,dwn,rst)
	begin
		if rst='0' then
			dir1	<= '0';
		elsif up='1' then
			dir1	<= '0';
		elsif dwn='1' then
			dir1	<= '1';
		end if;
	end process;

-- Display enable generation
	process(en_clk,rst)
	begin
		if rst='0' then 
			en_s	<= (0=>'1',others=>'0');
		 elsif en_clk'event and en_clk='1' then
		 	en_s 	<= en_s(0) & en_s(3 downto 1);
		  end if ;
		end process;


process(dir1,rst)
begin
	if rst='0' then 
		cntr1	<= (others=>'0');
	 elsif dwn'event and dwn='1' then
	 	if cntr1="1001" then
			cntr1	<= (others=>'0');
    	else
			cntr1 	<= cntr1 +1;
	    end if ;
    end if ;
end process;

process(dir2,rst)
begin
	if rst='0' then 
		cntr2	<= (others=>'0');
	 elsif dir2'event and dir2='0' then
	 	if cntr2="1001" then
			cntr2	<= (others=>'0');
    	else
			cntr2 	<= cntr2 +1;
	    end if ;
    end if ;
end process;


process(dir3,rst)
begin
	if rst='0' then 
		cntr3	<= (others=>'0');
	 elsif dir3'event and dir3='0' then
	 	if cntr3="1001" then
			cntr3	<= (others=>'0');
    	else
			cntr3 	<= cntr3 +1;
	    end if ;
    end if ;
end process;


process(dir4,rst)
begin
	if rst='0' then 
		cntr4	<= (others=>'0');
	 elsif dir4'event and dir4='0' then
	 	if cntr4="1001" then
			cntr4	<= (others=>'0');
    	else
			cntr4 	<= cntr4 +1;
	    end if ;
    end if ;
end process;

end Behavioral;
